@@ -1,5 +1,5 @@
 //
-// Copyright 2014 Ettus Research LLC
+// Copyright 2014-2015 Ettus Research LLC
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,11 +18,11 @@
 #ifndef INCLUDED_UHD_UTILS_MATH_HPP
 #define INCLUDED_UHD_UTILS_MATH_HPP
 
+#include <cmath>
 #include <uhd/config.hpp>
 #include <boost/cstdint.hpp>
 #include <boost/numeric/conversion/bounds.hpp>
 
-
 namespace uhd {
 
 /*!
@@ -237,10 +237,20 @@ namespace fp_compare {
                 == fp_compare::fp_compare_delta<double>(rhs, FREQ_COMPARISON_DELTA_HZ));
     }
 
+    //! Portable log2()
+    template <typename float_t> UHD_INLINE
+    float_t log2(float_t x)
+    {
+        // C++11 defines std::log2(), when that's universally supported
+        // we can switch over.
+        return std::log(x) / std::log(float_t(2));
+    }
+
+
 } // namespace math
 } // namespace uhd
 
-#include "fp_compare_epsilon.ipp"
-#include "fp_compare_delta.ipp"
+#include <uhd/utils/fp_compare_epsilon.ipp>
+#include <uhd/utils/fp_compare_delta.ipp>
 
 #endif /* INCLUDED_UHD_UTILS_MATH_HPP */
