@@ -1,5 +1,5 @@
 //
-// Copyright 2012-2013,2015 Ettus Research LLC
+// Copyright 2012-2013 Ettus Research LLC
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -27,40 +27,55 @@
 #include <boost/utility.hpp>
 #include "ad9361_ctrl.hpp"
 
-enum b200_type_t {
+enum b200_product_t {
     B200,
-    B210
+    B210,
+    B205
 };
 
+// These are actual USB PIDs (not Ettus Product IDs)
 const static boost::uint16_t B200_VENDOR_ID     = 0x2500;
 const static boost::uint16_t B200_VENDOR_NI_ID  = 0x3923;
 const static boost::uint16_t B200_PRODUCT_ID    = 0x0020;
+const static boost::uint16_t B205_PRODUCT_ID    = 0x0021;
 const static boost::uint16_t B200_PRODUCT_NI_ID = 0x7813;
 const static boost::uint16_t B210_PRODUCT_NI_ID = 0x7814;
 const static boost::uint16_t FX3_VID            = 0x04b4;
 const static boost::uint16_t FX3_DEFAULT_PID    = 0x00f3;
 const static boost::uint16_t FX3_REENUM_PID     = 0x00f0;
 
+//! Map the USB PID to the product (only for PIDs that map to a single product)
+static const uhd::dict<boost::uint16_t, b200_product_t> B2XX_PID_TO_PRODUCT = boost::assign::map_list_of
+        (B200_PRODUCT_NI_ID, B200)
+        (B210_PRODUCT_NI_ID, B210)
+        (B205_PRODUCT_ID,    B205)
+;
+
 static const std::string     B200_FW_FILE_NAME = "usrp_b200_fw.hex";
 
-//! Map the product ID (in the EEPROM) to a device type
-static const uhd::dict<boost::uint16_t, b200_type_t> B2X0_PRODUCT_ID = boost::assign::map_list_of
+//! Map the EEPROM product ID codes to the product
+static const uhd::dict<boost::uint16_t, b200_product_t> B2XX_PRODUCT_ID = boost::assign::map_list_of
         (0x0001,             B200)
         (0x7737,             B200)
         (B200_PRODUCT_NI_ID, B200)
         (0x0002,             B210)
         (0x7738,             B210)
         (B210_PRODUCT_NI_ID, B210)
+        (0x0003,             B205)
+        (0x7739,             B205)
 ;
 
-static const uhd::dict<b200_type_t, std::string> B2X0_STR_NAMES = boost::assign::map_list_of
+
+static const uhd::dict<b200_product_t, std::string> B2XX_STR_NAMES = boost::assign::map_list_of
         (B200, "B200")
         (B210, "B210")
+        (B205, "B200mini")
 ;
 
-static const uhd::dict<b200_type_t, std::string> B2X0_FPGA_FILE_NAME = boost::assign::map_list_of
+static const uhd::dict<b200_product_t, std::string> B2XX_FPGA_FILE_NAME = boost::assign::map_list_of
         (B200, "usrp_b200_fpga.bin")
         (B210, "usrp_b210_fpga.bin")
+        (B205, "usrp_b200mini_fpga.bin")
 ;
 
 
@@ -97,7 +112,7 @@ public:
     virtual void set_fpga_reset_pin(const bool reset) = 0;
 
     //! load an FPGA image
-    virtual boost::uint32_t load_fpga(const std::string filestring) = 0;
+    virtual boost::uint32_t load_fpga(const std::string filestring, bool force=false) = 0;
 
     virtual void write_eeprom(boost::uint16_t addr, boost::uint16_t offset, const uhd::byte_vector_t &bytes) = 0;
 
